from graphene_django.filter import DjangoFilterConnectionField
from graphene_permissions.permissions import AllowAny


class AuthNode:
    permission_classes = (AllowAny,)

    @classmethod
    def get_node(cls, info, id):
        if all([perm().has_node_permission(info, id) for perm in cls.permission_classes]):
            try:
                object_instance = cls._meta.model.objects.get(id=id)
            except cls._meta.model.DoesNotExist:
                object_instance = None
            return object_instance
        else:
            return None


class AuthMutation:
    permission_classes = (AllowAny,)

    @classmethod
    def has_permission(cls, root, info, input):
        return all(
            [perm().has_mutation_permission(root, info, input) for perm in cls.permission_classes]
        )


class AuthFilter(DjangoFilterConnectionField):
    """
    Custom ConnectionField for basic authentication system.
    """
    permission_classes = (AllowAny,)

    @classmethod
    def has_permission(cls, info):
        return all(
            [perm().has_filter_permission(info) for perm in cls.permission_classes]
        )

    @classmethod
    def connection_resolver(
            cls, resolver, connection, default_manager, max_limit,
            enforce_first_or_last, filterset_class, filtering_args,
            root, info, **args
    ):

        filter_kwargs = {k: v for k, v in args.items() if k in filtering_args}
        qs = filterset_class(
            data=filter_kwargs,
            queryset=default_manager.get_queryset()
        ).qs

        if not cls.has_permission(info):
            return super(DjangoFilterConnectionField, cls).connection_resolver(
                resolver, connection, qs.none(),
                max_limit, enforce_first_or_last, root, info, **args
            )

        return super(DjangoFilterConnectionField, cls).connection_resolver(
            resolver, connection, qs, max_limit, enforce_first_or_last,
            filterset_class, filtering_args, **args,
        )



from graphene_permissions import __version__
from pip.download import PipSession
from pip.req import parse_requirements
from setuptools import setup

with open('README.md', encoding="utf-8") as f:
    long_description = f.read()

install_requirements = parse_requirements('requirements/requirements.txt', session=PipSession())

setup(
    name='graphene-permissions',
    packages=('graphene_permissions',),
    license='MIT',
    version=__version__,
    author='redzej',
    description='Simple graphene-django permission system.',
    long_description=long_description,
    url='https://github.com/redzej/graphene-permissions',
    install_requires=[str(ir.req) for ir in install_requirements],
    keywords='graphene django permissions permission system',
    python_requires='>=3.5',
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Topic :: Internet :: WWW/HTTP',
        'Intended Audience :: Developers',
    ),
)



class AllowAny:
    """
    Default authentication class.
    Allows any user for any action.
    Subclass it and override methods below.
    """

    @staticmethod
    def has_node_permission(info, id):
        return True

    @staticmethod
    def has_mutation_permission(root, info, input):
        return True

    @staticmethod
    def has_filter_permission(info):
        return True


class AllowAuthenticated:
    """
    Allows performing action only for logged in users.
    """

    @staticmethod
    def has_node_permission(info, id):
        return info.context.user.is_authenticated

    @staticmethod
    def has_mutation_permission(root, info, input):
        return info.context.user.is_authenticated

    @staticmethod
    def has_filter_permission(info):
        return info.context.user.is_authenticated


class AllowStaff:
    """
    Allow performing action only for staff users.
    """

    @staticmethod
    def has_node_permission(info, id):
        return info.context.user.is_staff

    @staticmethod
    def has_mutation_permission(root, info, input):
        return info.context.user.is_staff

    @staticmethod
    def has_filter_permission(info):
        return info.context.user.is_staff
